﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class scr_Spawner : MonoBehaviour {
	public Transform[] SpawnerArray; //Array of Spawn Locations
	public GameObject Mine;
	public GameObject enemySub;
	public float mineCooldown = 0.5f;
	public float subCooldown = 3.0f;

	private float mineUpTime = 0.0f;
	private float subUpTime = 0.0f;
	public int numberOfMines = 0;
	public int numberOfSubs = 0; 

	public bool finishedSpawningMines = true;
	public bool finishedSpawningSubs = true;

	//This class is given a mine object and a list of spawn locations
	//It's job is to spawn the required number of bombs with an initial velocity using a timer
	void Update() {
		//Every frame, we are counting the amount of time that has passed, 
		//and testing if we are ready to spawn another mine
		mineUpTime += Time.deltaTime; subUpTime += Time.deltaTime;
		if (mineUpTime > mineCooldown && !finishedSpawningMines) {
				numberOfMines-=1; //decrement the number of mines
				if (numberOfMines <= 0) {
					finishedSpawningMines = true;
				}
				mineUpTime = 0.0f; //reset the upTime
				spawnMine (); //spawn a mine
		}
		if (subUpTime > subCooldown && !finishedSpawningSubs) {
			numberOfSubs-=1; //decrement the number of mines
			if (numberOfSubs <= 0) {
				finishedSpawningSubs = true;
			}
			mineUpTime = 0.0f; //reset the upTime
			spawnSub (); //spawn a mine
		}
	}

	/**
	 * Helper method to spawn a mine at a RANDOM index in the spawner array
	 * */
	private void spawnMine() {
		int spawnIndex = (int) Random.Range (0.0f, 8.0f); //Get a random spawn point
		Transform spawnPoint = SpawnerArray [spawnIndex]; //Assign its transform to a temp variable
		Transform spawnedMine = Instantiate(Mine, spawnPoint.position, spawnPoint.rotation, transform).transform; 
		//Instantiate the mine at the spawnpoint and add it to the mine array
	}

	/*
	 * Helper methiod to spawn a sub at a RANDOM index
	 * */
	private void spawnSub() {
		Debug.Log ("Spawning a sub!");
		int spawnIndex = (int) Random.Range (0.0f, 8.0f); //Get a random spawn point
		Transform spawnPoint = SpawnerArray [spawnIndex]; //Assign its transform to a temp variable
		Transform spawnedMine = Instantiate(enemySub, spawnPoint.position, spawnPoint.rotation, transform).transform; 
		//Instantiate the mine at the spawnpoint and add it to the mine array
	}

	/**
	 * Method to destroy all mines on the field
	 * */
	public void destroyAll() {
		GameObject[] spawnedMines = GameObject.FindGameObjectsWithTag ("Mine");
		for (int i = 0; i < spawnedMines.Length; i++) {
			Destroy (spawnedMines [i]);
		}
		GameObject[] spawnedSubs = GameObject.FindGameObjectsWithTag ("Enemy");
		for (int i = 0; i < spawnedSubs.Length; i++) {
			Destroy (spawnedSubs [i]);
		}
	}

	/**
	 * Method to start spawning mines given a count and cooldown
	 * */
	public void startSpawningMines(int mineCount, float newCooldown) {
		numberOfMines = mineCount;
		finishedSpawningMines = false;
		mineUpTime = 0.0f;
		mineCooldown = newCooldown;
	}
	/**
	 * Method to start spawning subs given a count and cooldown
	 * */
	public void startSpawningSubs(int subCount, float newCooldown) {
		numberOfSubs = subCount;
		finishedSpawningSubs = false;
		subUpTime = 0.0f;
		subCooldown = newCooldown;
	}

	/**
	 * Stop the mines and subs from spawning
	 * */
	public void stopSpawning() {
		finishedSpawningMines = true;
		finishedSpawningSubs = true;
		mineUpTime = 0.0f; subUpTime = 0.0f;
	}
}
