﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

//Follow the player until you reach a certain radius
//Stop and attempt to shoot player
public class scr_enemy : MonoBehaviour {
	public float health = 3.0f;
	public float minimumDistance = 5.0f;
	public float speed = 0.065f;
	public float fireCooldown = 3.0f;
	public float stopCooldown = 5.0f;
	public GameObject model;
	public AudioClip hit;
	public AudioClip death;
	public GameObject explosion;
	public Transform projectile;
	public Transform projectileSpawn;

	private scr_gameManager manager;
	private bool dead = false;
	private AudioSource audioSource;
	private float fireUpTime = 0.0f;
	private float stopUpTime = 0.0f;
	private Transform playerPos;
	private float distance = 0.0f;
	private scr_player player;
	private Rigidbody rb;
	private CapsuleCollider myColl;

	void Start() {
		player = FindObjectOfType<scr_player> (); 
		rb = GetComponent<Rigidbody> ();
		audioSource = GetComponent<AudioSource> ();
		myColl = GetComponent<CapsuleCollider> ();
		manager = FindObjectOfType<scr_gameManager> ();
	}

	void Update() {
		//update the location of the player
		if (!dead) {
			updateTarget();
			addToTimers ();
			//is the target within range?
			if (withinRange ()) {
				//try to fire at target
				fireAtTarget();
			} else {
				//move towards the target
				moveToTarget();
			}
		}
	}

	//add to any timers for cooldown
	private void addToTimers() {
		fireUpTime += Time.deltaTime;
		stopUpTime += Time.deltaTime;
	}

	/**
	 * Method to test if we are ready to fire
	 * */
	private bool readyToFire() {
		return (fireUpTime > fireCooldown);
	}

	/**
	 * Method to fire at the target when in range
	 * */
	private void fireAtTarget() {
		rb.velocity = Vector3.zero;
		if (readyToFire ()) {
			Instantiate (projectile, 
				projectileSpawn.position, projectileSpawn.rotation);
			Debug.Log ("I am firing at ze player!@!!");
			fireUpTime = 0.0f;
		} else {
			transform.LookAt (playerPos);
		}
	}

	private void moveToTarget() {
		Debug.Log ("I am moving towards target");
		if (stopUpTime > stopCooldown) {
			rb.velocity = Vector3.zero;
			stopUpTime = 0.0f;
		} else {
			transform.LookAt (playerPos);
			rb.AddForce (transform.forward.normalized * speed * 0.1f, ForceMode.VelocityChange);
		}
	}

	/**
	 * Check if the player is within our minimum range
	 * */
	private bool withinRange() {
		return (distance <= minimumDistance);
	}

	/**
	 * Update the position of the player
	 * */
	private void updateTarget() {
		playerPos = player.transform;
		distance = Vector3.Distance (transform.position, playerPos.position);
	}

	/**
	 * This can be changed later
	 * */
	private void takeDamage() {
		health -= 1.0f;
		audioSource.clip = hit;
		audioSource.Play ();
	}

	/**
	 * Take damage when enemy missile collides with us
	 * */
	void OnTriggerEnter(Collider other) {
		if (other.CompareTag ("PlayerWeapon")) {
			takeDamage ();
			//Needs sound and explosion
			if (health <= 0.0f) {
				dead = true;
				manager.tempSubCount--;
				myColl.enabled = false;
				model.SetActive (false);
				audioSource.clip = death;
				audioSource.Play ();
				Instantiate(explosion, transform.position, transform.rotation, transform);
				Debug.Log ("I am le dead");
				Destroy (gameObject, 1.5f);
			}
		}
	}
}
