﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class scr_enemySub : MonoBehaviour {

    public float accel;
    public float topSpeed;
    public float turnSpeed;
    public float discoverDistance;
    public float scanAngle;
    public float scanAccel;
    public float scanTopSpeed;
    public GameObject model;
    public GameObject explosion;


    public float deathTime = 1.5f;

    private GameObject Player;
    private Transform playerPos;
    private Rigidbody enemyRigidBody;
    private Collider capsuleCollider;


    private Transform originalTrans;
    private Quaternion outerTurnL;
    private Quaternion outerTurnR;
    private enum TurningMode { LEFT, RIGHT, FIRST };
    private TurningMode tm;

    private AudioSource audioSource;

    private const float TURN_MARGIN = 10f;

	// Use this for initialization
	void Start () {
        Player = GameObject.FindGameObjectWithTag("Player");
        playerPos = Player.transform;
        enemyRigidBody = GetComponent<Rigidbody>();
        audioSource = GetComponent<AudioSource>();
        capsuleCollider = GetComponent<CapsuleCollider>();


        originalTrans = transform;
        outerTurnL = originalTrans.rotation;
        outerTurnL.y += scanAngle;
        outerTurnR = originalTrans.rotation;
        outerTurnR.y -= scanAngle;
        tm = TurningMode.FIRST;
        //Debug.Log("outerTurnL:" + outerTurnL.y);
        //Debug.Log("outerTurnR:" + outerTurnR.y);
	}
	
	// Update is called once per frame
	void Update () {
        playerPos = Player.transform;
        Move();
        
        // If player is in view, shoot
        RaycastHit hit;
        Ray forwardRay = new Ray(transform.position, transform.forward);
        if (Physics.Raycast(forwardRay, out hit, discoverDistance))
        {
            if (hit.collider.tag == "Player")
            {
                //Debug.Log("I see the player!");

                /*
                 *  INSERT SHOOTING MECHANISM HERE
                 *  
                 *  Note: this ray is cast every frame and if the enemy is staring right at the player,
                 *  there will be a lot of successful hits. The torpedo shooting should have some cooldown,
                 *  ie, probably shouldn't shoot a torpedo for every ray that hits.
                 *  
                 * */

            }
        }
	}

    void Move()
    {
        Vector3 movVector = transform.forward.normalized;
        // Check if player is within the discover distance
        if (Mathf.Abs(Vector3.Distance(transform.position, playerPos.position)) < discoverDistance)
        {
            Quaternion targetRotation = Quaternion.LookRotation(playerPos.transform.position - transform.position);         
            transform.rotation = Quaternion.Slerp(transform.rotation, targetRotation, turnSpeed * Time.deltaTime);

            movVector.Scale(new Vector3(accel, accel, accel));
            if (Mathf.Abs(Vector3.Distance(transform.position, playerPos.position)) < discoverDistance / 0.9)
            {
                if (enemyRigidBody.velocity.x < topSpeed && enemyRigidBody.velocity.y < topSpeed && enemyRigidBody.velocity.z < topSpeed)
                    enemyRigidBody.AddForce(movVector);
            }
        }
        // If moving and do not see player, slow to halt
        else if (enemyRigidBody.velocity != Vector3.zero)
        {
            enemyRigidBody.velocity = Vector3.Slerp(enemyRigidBody.velocity, Vector3.zero, Time.deltaTime * turnSpeed);
        }
        // No longer moving, perform idle
        else
        {
            IdleMovement();
        }

    }

    void IdleMovement()
    {
        // Check for full turns
        if (Mathf.Abs(enemyRigidBody.rotation.eulerAngles.y - outerTurnL.y) < TURN_MARGIN && tm != TurningMode.RIGHT)
        {
            //Debug.Log("Finished rotation to the right");
            tm = TurningMode.RIGHT;
        }
        else if (Mathf.Abs(enemyRigidBody.rotation.eulerAngles.y - outerTurnR.y - 360) < TURN_MARGIN && tm != TurningMode.LEFT)
        {
            //Debug.Log("Finished rotation to the left");
            tm = TurningMode.LEFT;
        }
        else 
        {
            if (tm == TurningMode.LEFT || tm == TurningMode.FIRST)
            {
                if (enemyRigidBody.angularVelocity.y < scanTopSpeed)
                    enemyRigidBody.AddTorque(0.0f, Time.deltaTime * scanAccel, 0.0f);
            }
            else if (tm == TurningMode.RIGHT)
            {
                if (enemyRigidBody.angularVelocity.y < scanTopSpeed)
                    enemyRigidBody.AddTorque(0.0f, -Time.deltaTime * scanAccel, 0.0f);
            }
        }
    }

    void OnTriggerEnter(Collider other)
    {
        audioSource.Play();
        model.SetActive(false);
        capsuleCollider.enabled = false;
        Instantiate(explosion, transform.position, transform.rotation, transform);
        Object.Destroy(gameObject, deathTime);
    } 
}
