﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class scr_gameManager : MonoBehaviour {
	private scr_player Player;
	private scr_Spawner Spawner;
	public int round;
	public AudioClip menuMusic;
	public AudioClip gameMusic;
	public Text UI_heath;
	public Text UI_score;
	public Text UI_mineCount;
	public Text UI_subCount;
	public Text UI_round;
	public GameObject UI_start;
	public GameObject UI_gameOver;

	private bool gamePlaying = false;
	private AudioSource audioSource;
	public int mineCount = 30;
	public int subCount = 3;
	public int tempSubCount = 3;
	public int tempMineCount = 30;
	public float mineDelay = 1.0f;
	public float subDelay = 10.0f;
	private bool freshStart = true;
	private float gameOverDelay = 0.75f;
	private float gameOverTime = 0.0f;

	// Use this for initialization
	void Start () {
		Debug.Log ("Hit space to play!");
		Player = FindObjectOfType<scr_player> ();
		Spawner = FindObjectOfType<scr_Spawner> ();
		audioSource = GetComponent<AudioSource> ();
		round = 0;
	}
	
	// Update is called once per frame
	void Update () {
		updateUI ();
		if (isGameOver()) {
			if (!freshStart) {
				gameOverTime += Time.deltaTime;
				if (gameOverTime > gameOverDelay) {
					UI_gameOver.GetComponent<Image>().enabled = true;
				}
			}
			//display a replay button
			if (Input.GetKeyDown (KeyCode.Space)) {
				gameOverTime = 0.0f;
				freshStart = false;
				UI_gameOver.GetComponent<Image>().enabled = false;
				UI_start.SetActive(false);
				gamePlaying = true;
				reset ();
				round++;
				Debug.Log ("Starting new game!");
				if (!audioSource.isPlaying)
					audioSource.Play ();
			}
			//play the menu music
			if (gamePlaying && audioSource.isPlaying) {
				gamePlaying = false;
				audioSource.Stop ();
				audioSource.clip = menuMusic;
				audioSource.Play ();
			}
		} else {
			if (!gamePlaying && audioSource.isPlaying) {
				gamePlaying = true;
				audioSource.Stop ();
				audioSource.clip = gameMusic;
				audioSource.Play ();
			}
			//check if the round is over
			if (isRoundOver()) {
				//reset the playing field
				amplify();
				Debug.Log ("Starting round "+round+"!");
				Spawner.startSpawningMines (mineCount, mineDelay); //Start spawning again
				Spawner.startSpawningSubs(subCount, subDelay);
			} //else do nothing
		}
	}

	/**
	 * Check if its gameOver
	 * */
	private bool isGameOver() {
		if (Player.health <= 0)
			return true;
		else
			return false;
	}

	/**
	 * Check if the round is over
	 * */
	public bool isRoundOver() {
		if (tempSubCount == 0 && tempMineCount == 0) {
			return true;
		} else
			return false;
	}

	/**
	 * Reset EVERYTHING
	 * */
	public void reset() {
		//reset all parameters for mine spawner and boat spawner
		Player.reset ();
		mineCount = 30; tempMineCount = mineCount; subCount = 3; tempSubCount = 3;
		mineDelay = 1.0f; subDelay = 10.0f;
		round = 0;
		Spawner.destroyAll();
		Spawner.startSpawningMines (mineCount, mineDelay);
		Spawner.startSpawningSubs (subCount, subDelay);
	}

	/**
	 * Make the round harder!
	 * */
	public void amplify() {
		mineCount *= 2; tempMineCount = mineCount;
		subCount *= 2; tempSubCount = subCount;
		mineDelay /= 2; mineDelay += 0.5f;
		round++;
	}

	/**
	 * 
	 * */
	public void updateUI() {
		UI_heath.text = "" + Player.health;
		UI_mineCount.text = "Mines: " + tempMineCount;
		UI_subCount.text = "Subs: " + tempSubCount;
		UI_round.text = "Round " + round;
		UI_score.text = "Score: " + Player.Score;
	}
}
