﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(scr_rbMover))]
[RequireComponent(typeof(scr_rotation))]
public class scr_inputHandler : MonoBehaviour {
	private scr_rbMover mover;
	private scr_player player;
	private scr_weapon weapon;
	private scr_rotation rotator;
	private float upTime = 0.0f;

	void Start () {
		mover = GetComponentInChildren<scr_rbMover> ();
		player = GetComponent<scr_player> ();
		weapon = GetComponent<scr_weapon> ();
		rotator = GetComponent<scr_rotation> ();
	}

	void Update () { 
		if (Input.GetKeyDown (KeyCode.Escape)) {
			Application.Quit ();
		}
		if (player.canMove && player.health > 0) {
			checkMovement ();
			checkBreaks ();
			checkFire ();
		}
	}

	/**
	 * Helper method to detect movement and apply it using
	 * a rbMover and speed param
	 * */
	private void checkMovement() {
		float _xRot = Input.GetAxisRaw ("Horizontal") * Time.fixedDeltaTime; //Get the horizontal movement (left right)
		float _forwardMovement = Input.GetAxisRaw ("Vertical")* Time.fixedDeltaTime;  //Get the vertical (forward back)

		//Rotate the player
		if (_xRot > 0 || _xRot < 0) {
			rotator.rotate (_xRot * player.rotSpeed);
		}
		//Move the player forward
		if (_forwardMovement > 0 || _forwardMovement < 0) {
			mover.move (_forwardMovement, player.speed);
		}
	}

	/**
	 * Helper method to detect input for firing missiles
	 * and apply it using a weapon
	 * */
	private void checkBreaks() {
		if (Input.GetKeyDown (KeyCode.Space)) {
			mover.stop ();
			//TODO to make this better, make applying the breaks prevent movement foreward but allow rotation
			//TODO using addforce, come to a stop instead of halting the player
		}
	}

	private void checkFire() {
		upTime += Time.deltaTime;
		if (Input.GetKeyDown (KeyCode.F) && upTime > player.fireDelay) {
			weapon.spawnProjectile (player.projectile, player.projSpawn);
			upTime = 0.0f;
		}
	}
}